# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/data/point.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from .. import options_pb2 as blickfeld_dot_options__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/data/point.proto',
  package='blickfeld.protocol.data',
  syntax='proto2',
  serialized_options=_b('\370\001\001'),
  serialized_pb=_b('\n\x1a\x62lickfeld/data/point.proto\x12\x17\x62lickfeld.protocol.data\x1a\x17\x62lickfeld/options.proto\"\xa6\x04\n\x05Point\x12\x10\n\x02id\x18\x01 \x01(\rB\x04\xb0\xb5\x18\x01\x12<\n\x07returns\x18\x02 \x03(\x0b\x32%.blickfeld.protocol.data.Point.ReturnB\x04\xb0\xb5\x18\x01\x12\x1d\n\x0fstart_offset_ns\x18\x03 \x01(\x04\x42\x04\xb0\xb5\x18\x01\x12!\n\x13\x61mbient_light_level\x18\x04 \x01(\rB\x04\xb0\xb5\x18\x01\x12\x41\n\tdirection\x18\x05 \x01(\x0b\x32(.blickfeld.protocol.data.Point.DirectionB\x04\xb0\xb5\x18\x01\x12\x18\n\nchannel_id\x18\x06 \x01(\rB\x04\xb0\xb5\x18\x01\x12\x43\n\x0b\x65rror_flags\x18\x07 \x03(\x0e\x32(.blickfeld.protocol.data.Point.ErrorFlagB\x04\xb0\xb5\x18\x01\x1a\x63\n\x06Return\x12\x10\n\x02id\x18\x01 \x01(\rB\x04\xb0\xb5\x18\x01\x12\x19\n\tcartesian\x18\x02 \x03(\x02\x42\x06\x10\x01\xb0\xb5\x18\x01\x12\x13\n\x05range\x18\x03 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x12\x17\n\tintensity\x18\x04 \x01(\rB\x04\xb0\xb5\x18\x01\x1aS\n\tDirection\x12\x15\n\x07\x61zimuth\x18\x01 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x12\x17\n\televation\x18\x02 \x01(\x02\x42\x04\xb0\xb5\x18\x01\x12\x16\n\x06origin\x18\x03 \x03(\x02\x42\x06\x10\x01\xb0\xb5\x18\x01\"/\n\tErrorFlag\x12\"\n\x1e\x45_VIBRATION_THRESHOLD_EXCEEDED\x10\x01\x42\x03\xf8\x01\x01')
  ,
  dependencies=[blickfeld_dot_options__pb2.DESCRIPTOR,])



_POINT_ERRORFLAG = _descriptor.EnumDescriptor(
  name='ErrorFlag',
  full_name='blickfeld.protocol.data.Point.ErrorFlag',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='E_VIBRATION_THRESHOLD_EXCEEDED', index=0, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=584,
  serialized_end=631,
)
_sym_db.RegisterEnumDescriptor(_POINT_ERRORFLAG)


_POINT_RETURN = _descriptor.Descriptor(
  name='Return',
  full_name='blickfeld.protocol.data.Point.Return',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='blickfeld.protocol.data.Point.Return.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cartesian', full_name='blickfeld.protocol.data.Point.Return.cartesian', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='range', full_name='blickfeld.protocol.data.Point.Return.range', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='intensity', full_name='blickfeld.protocol.data.Point.Return.intensity', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=398,
  serialized_end=497,
)

_POINT_DIRECTION = _descriptor.Descriptor(
  name='Direction',
  full_name='blickfeld.protocol.data.Point.Direction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='azimuth', full_name='blickfeld.protocol.data.Point.Direction.azimuth', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='elevation', full_name='blickfeld.protocol.data.Point.Direction.elevation', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='origin', full_name='blickfeld.protocol.data.Point.Direction.origin', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\020\001\260\265\030\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=499,
  serialized_end=582,
)

_POINT = _descriptor.Descriptor(
  name='Point',
  full_name='blickfeld.protocol.data.Point',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='blickfeld.protocol.data.Point.id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='returns', full_name='blickfeld.protocol.data.Point.returns', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_offset_ns', full_name='blickfeld.protocol.data.Point.start_offset_ns', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ambient_light_level', full_name='blickfeld.protocol.data.Point.ambient_light_level', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='direction', full_name='blickfeld.protocol.data.Point.direction', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channel_id', full_name='blickfeld.protocol.data.Point.channel_id', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_flags', full_name='blickfeld.protocol.data.Point.error_flags', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=_b('\260\265\030\001'), file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_POINT_RETURN, _POINT_DIRECTION, ],
  enum_types=[
    _POINT_ERRORFLAG,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=631,
)

_POINT_RETURN.containing_type = _POINT
_POINT_DIRECTION.containing_type = _POINT
_POINT.fields_by_name['returns'].message_type = _POINT_RETURN
_POINT.fields_by_name['direction'].message_type = _POINT_DIRECTION
_POINT.fields_by_name['error_flags'].enum_type = _POINT_ERRORFLAG
_POINT_ERRORFLAG.containing_type = _POINT
DESCRIPTOR.message_types_by_name['Point'] = _POINT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Point = _reflection.GeneratedProtocolMessageType('Point', (_message.Message,), dict(

  Return = _reflection.GeneratedProtocolMessageType('Return', (_message.Message,), dict(
    DESCRIPTOR = _POINT_RETURN,
    __module__ = 'blickfeld.data.point_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.Point.Return)
    ))
  ,

  Direction = _reflection.GeneratedProtocolMessageType('Direction', (_message.Message,), dict(
    DESCRIPTOR = _POINT_DIRECTION,
    __module__ = 'blickfeld.data.point_pb2'
    # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.Point.Direction)
    ))
  ,
  DESCRIPTOR = _POINT,
  __module__ = 'blickfeld.data.point_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.data.Point)
  ))
_sym_db.RegisterMessage(Point)
_sym_db.RegisterMessage(Point.Return)
_sym_db.RegisterMessage(Point.Direction)


DESCRIPTOR._options = None
_POINT_RETURN.fields_by_name['id']._options = None
_POINT_RETURN.fields_by_name['cartesian']._options = None
_POINT_RETURN.fields_by_name['range']._options = None
_POINT_RETURN.fields_by_name['intensity']._options = None
_POINT_DIRECTION.fields_by_name['azimuth']._options = None
_POINT_DIRECTION.fields_by_name['elevation']._options = None
_POINT_DIRECTION.fields_by_name['origin']._options = None
_POINT.fields_by_name['id']._options = None
_POINT.fields_by_name['returns']._options = None
_POINT.fields_by_name['start_offset_ns']._options = None
_POINT.fields_by_name['ambient_light_level']._options = None
_POINT.fields_by_name['direction']._options = None
_POINT.fields_by_name['channel_id']._options = None
_POINT.fields_by_name['error_flags']._options = None
# @@protoc_insertion_point(module_scope)
