# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blickfeld/config/secure.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='blickfeld/config/secure.proto',
  package='blickfeld.protocol.config',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x1d\x62lickfeld/config/secure.proto\x12\x19\x62lickfeld.protocol.config\"\x9c\x02\n\x06Secure\x12\x42\n\x0bpermissions\x18\x01 \x03(\x0e\x32-.blickfeld.protocol.config.Secure.Permissions\x12=\n\tcert_type\x18\x02 \x01(\x0e\x32*.blickfeld.protocol.config.Secure.CertType\x12\x15\n\rversion_major\x18\x03 \x01(\r\x12\x15\n\rversion_minor\x18\x04 \x01(\r\x12\x15\n\rversion_patch\x18\x05 \x01(\r\"(\n\x0bPermissions\x12\n\n\x06PUBLIC\x10\x00\x12\r\n\tDEVELOPER\x10\x01\" \n\x08\x43\x65rtType\x12\n\n\x06\x43LIENT\x10\x00\x12\x08\n\x04\x43UBE\x10\x01')
)



_SECURE_PERMISSIONS = _descriptor.EnumDescriptor(
  name='Permissions',
  full_name='blickfeld.protocol.config.Secure.Permissions',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PUBLIC', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEVELOPER', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=271,
  serialized_end=311,
)
_sym_db.RegisterEnumDescriptor(_SECURE_PERMISSIONS)

_SECURE_CERTTYPE = _descriptor.EnumDescriptor(
  name='CertType',
  full_name='blickfeld.protocol.config.Secure.CertType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CLIENT', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CUBE', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=313,
  serialized_end=345,
)
_sym_db.RegisterEnumDescriptor(_SECURE_CERTTYPE)


_SECURE = _descriptor.Descriptor(
  name='Secure',
  full_name='blickfeld.protocol.config.Secure',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='permissions', full_name='blickfeld.protocol.config.Secure.permissions', index=0,
      number=1, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cert_type', full_name='blickfeld.protocol.config.Secure.cert_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version_major', full_name='blickfeld.protocol.config.Secure.version_major', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version_minor', full_name='blickfeld.protocol.config.Secure.version_minor', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='version_patch', full_name='blickfeld.protocol.config.Secure.version_patch', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SECURE_PERMISSIONS,
    _SECURE_CERTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=61,
  serialized_end=345,
)

_SECURE.fields_by_name['permissions'].enum_type = _SECURE_PERMISSIONS
_SECURE.fields_by_name['cert_type'].enum_type = _SECURE_CERTTYPE
_SECURE_PERMISSIONS.containing_type = _SECURE
_SECURE_CERTTYPE.containing_type = _SECURE
DESCRIPTOR.message_types_by_name['Secure'] = _SECURE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Secure = _reflection.GeneratedProtocolMessageType('Secure', (_message.Message,), dict(
  DESCRIPTOR = _SECURE,
  __module__ = 'blickfeld.config.secure_pb2'
  # @@protoc_insertion_point(class_scope:blickfeld.protocol.config.Secure)
  ))
_sym_db.RegisterMessage(Secure)


# @@protoc_insertion_point(module_scope)
